﻿
XIncludeFile "!pack.pbi"

DeclareModule JSON
  EnableExplicit
  Declare Pack(file.s="")
  Declare Open(file.s)
  Declare Save(file.s)
  Declare Close()
  Declare.s GetString(key.s="",def.s="")
  Declare GetBool(key.s="",def=0)
  Declare GetInteger(key.s="",def=0)
  Declare.d GetDouble(key.s="",def.d=0)
  Declare GetStringList(key.s,List string.s())
  
  Declare OpenSub(key.s="")
  Declare CreateSub(key.s)
  Declare CloseSub()
  
  Declare Examine(key.s="")
  Declare NextEntry()
  Declare Finish()
  
  Declare OpenArray(key.s="")
  Declare NextArrayEntry()
  Declare CloseArray()
  
  Global Entry.s 
  
  
  Declare New()
  Declare.s Output()
  Declare Export()
  Declare SetBool(key.s,bool)
  Declare SetInteger(key.s,Integer)
  Declare SetDouble(key.s,Double.d)
  Declare SetString(key.s,string.s)
  
  Declare CreateArray(key.s="")
  Declare AddArrayEntry()
  
EndDeclareModule
Module JSON
  Global json,object
  
  Enumeration
    #back_none
    #back_array
    #back_sub
    #back_examine
  EndEnumeration
  
  Macro CheckError(typ)
    CompilerIf #PB_Compiler_Debugger
      If backlist()\type<>typ
        Debug #PB_Compiler_Procedure+" - Error!"
        Select backlist()\type
          Case #back_none :Debug "Should be none"
          Case #back_array : Debug "Should array"
          Case #back_sub : Debug "should sub"
          Case #back_examine: Debug "should examine"
          Default : Debug "unknown"
        EndSelect
        CallDebugger
      EndIf
    CompilerEndIf
  EndMacro
  
  
  Structure sBacklist
    object.i
    type.i
  EndStructure
  
  Global NewList backlist.sBacklist()
  
  Procedure Save(file.s)
    SaveJSON(json,file,#PB_JSON_PrettyPrint)
  EndProcedure  
  Procedure Open(file.s)
    If json
      FreeJSON(json)
    EndIf
    json=LoadJSON(#PB_Any,file)
    If json
      ClearList(backlist())      
      object=JSONValue(json)
      AddElement(backlist())
      backlist()\object=object
      backlist()\type=#back_none
    EndIf
    ProcedureReturn json
  EndProcedure
  Procedure Pack(file.s="")
    If Json
      FreeJSON(json)
    EndIf
    json=pack::GetJson(file)    
    If json
      ClearList(backlist())      
      object=JSONValue(json)
      AddElement(backlist())
      backlist()\object=object
      backlist()\type=#back_none
    EndIf
    ProcedureReturn json
  EndProcedure
  Procedure New()
    If json
      FreeJSON(json)
    EndIf
    json=CreateJSON(#PB_Any)
    If json
      ClearList(backlist())
      object=JSONValue(json)
      SetJSONObject(object)
      AddElement(backlist())
      backlist()\object=object
      backlist()\type=#back_none
    EndIf
    ProcedureReturn json
  EndProcedure
  Procedure.s Output()
    ProcedureReturn ComposeJSON(json,#PB_JSON_PrettyPrint)
  EndProcedure
  Procedure Export()
    Protected *buf,size,exsize
    size=ExportJSONSize(json,#PB_JSON_PrettyPrint)
    If size
      *buf=AllocateMemory(size)
      If *buf
        exsize= ExportJSON(json,*buf,size,#PB_JSON_PrettyPrint)
        If exsize<>size
          Debug "" +exsize+" "+size
          FreeMemory(*buf)
          *buf=#Null
        EndIf      
      EndIf
    EndIf
    ProcedureReturn *buf
  EndProcedure
  
  Procedure Close()
    If json
      CheckError(#back_none)
      DeleteElement(backlist())
      CompilerIf #PB_Compiler_Debugger
        If ListSize(backlist())
          Debug "Backlist:"+ListSize(backlist())
          While ListIndex(backlist())>=0
            Select backlist()\type
              Case #back_none :Debug "Missing none"
              Case #back_array : Debug "Missing array"
              Case #back_sub : Debug "Missing sub"
              Case #back_examine: Debug "Missing examine"
            EndSelect
            DeleteElement(backlist())
          Wend
        EndIf
      CompilerEndIf
        
      FreeJSON(json)
      json=0
    EndIf
  EndProcedure
  
  Procedure OpenSub(key.s="")
    Protected sub
    If key
      sub = GetJSONMember(object,key)
    Else
      sub=object
    EndIf
    If sub And JSONType(sub)=#PB_JSON_Object
      AddElement(backlist())
      backlist()\object=object
      backlist()\type=#back_sub
      object=sub
      ProcedureReturn #True
    EndIf
    ProcedureReturn #False
  EndProcedure
  Procedure CreateSub(key.s)
    Protected sub
    sub=AddJSONMember(object,key)
    SetJSONObject(sub)
    
    AddElement(backlist())
    backlist()\object=object
    backlist()\type=#back_sub
    object=sub
    ProcedureReturn #True
  EndProcedure    
  
  Procedure CloseSub()
    object=backlist()\object
    CheckError(#back_sub)
    DeleteElement(backlist())
    ProcedureReturn #False
  EndProcedure
  
  Procedure Examine(key.s="")
    If key<>"" 
      If opensub(key)=#False
        ProcedureReturn #False
      EndIf
    Else
      AddElement(backlist())
      backlist()\object=object
      backlist()\type=#back_sub
    EndIf
    
    If JSONType(object)=#PB_JSON_Object And ExamineJSONMembers(object)
      AddElement(backlist())
      backlist()\object=object       
      backlist()\type=#back_examine
      ProcedureReturn #True
    EndIf
       
    CloseSub()
    ProcedureReturn #False
  EndProcedure
  Procedure NextEntry()
    object=backlist()\object
    CheckError(#back_examine)
    ;dont remove!
    If NextJSONMember(object)
      entry=JSONMemberKey(object)
      object=JSONMemberValue(object)
      ProcedureReturn #True
    EndIf
    ProcedureReturn #False
  EndProcedure
  Procedure Finish()
    CheckError(#back_examine)
    DeleteElement(backlist())
    closesub()
  EndProcedure
  
  Global ArrayId,ArrayCount
  
  
  Procedure OpenArray(key.s="")  
    Protected sub,type
    If key
      sub = GetJSONMember(object,key)
    Else
      sub=object
    EndIf
    If sub=0
      ProcedureReturn #False
    EndIf
    
    type=JSONType(sub)
    If type=#PB_JSON_Array Or type=#PB_JSON_Object
      AddElement(backlist())
      backlist()\object=object
      backlist()\type=#back_sub
      object=sub
      AddElement(backlist())
      backlist()\object=arrayid
      backlist()\type=ArrayCount
      AddElement(backlist())
      backlist()\object=object  
      backlist()\type=#back_array
      
      arrayid=-1
      If type=#PB_JSON_Array
        arraycount=JSONArraySize(object)
      Else
        ArrayCount=-1
      EndIf
      ProcedureReturn #True
    EndIf
    ProcedureReturn #False
  EndProcedure
  Procedure NextArrayEntry()
    object=backlist()\object
    CheckError(#back_array)
    
    ArrayId+1
    
    If ArrayCount=-1
      If arrayid=0
        ProcedureReturn object
      Else
        ProcedureReturn #False
      EndIf
    EndIf
    
    If ArrayId<ArrayCount
      object=GetJSONElement(object,arrayid)
      ProcedureReturn object
    EndIf
    ProcedureReturn #False
  EndProcedure
  Procedure CloseArray()  
    CheckError(#back_array)
    DeleteElement(backlist())
    arrayid=backlist()\object
    ArrayCount=backlist()\type
    DeleteElement(backlist())
    closesub()
  EndProcedure
  
  Procedure CreateArray(key.s="")
    Protected sub
    If key
      sub=AddJSONMember(object,key)
    Else
      sub=object
    EndIf
    SetJSONArray(sub)
    
    AddElement(backlist())
    backlist()\object=object
    backlist()\type=#back_sub
    object=sub
    AddElement(backlist())
    backlist()\object=arrayid
    backlist()\type=ArrayCount
    AddElement(backlist())
    backlist()\object=object  
    backlist()\type=#back_array
    
    arrayid=-1
    ProcedureReturn #True
      
  EndProcedure
  Procedure AddArrayEntry()
    object=backlist()\object
    CheckError(#back_array)
    ArrayId+1
    object=AddJSONElement(object)
    SetJSONObject(object)
    ProcedureReturn #True
  EndProcedure
  
  
  
  Procedure.s GetString(key.s="",def.s="")
    Protected sub 
    If key<>""
      sub= GetJSONMember(object,key)
    Else
      sub=object
    EndIf
    If sub And JSONType(sub)=#PB_JSON_String
      ProcedureReturn GetJSONString(sub)
    EndIf
    ProcedureReturn def
  EndProcedure
  Procedure GetBool(key.s="",def=0)
    Protected sub 
    If key<>""
      sub= GetJSONMember(object,key)
    Else
      sub=object
    EndIf
    If sub And JSONType(sub)=#PB_JSON_Boolean
      ProcedureReturn GetJSONBoolean(sub)
    EndIf
    ProcedureReturn def
  EndProcedure  
  Procedure GetInteger(key.s="",def=0)
    Protected sub 
    If key<>""
      sub= GetJSONMember(object,key)
    Else
      sub=object
    EndIf
    If sub And JSONType(sub)=#PB_JSON_Number
      ProcedureReturn GetJSONInteger(sub)
    EndIf
    ProcedureReturn def
  EndProcedure
  Procedure.d GetDouble(key.s="",def.d=0)
    Protected sub 
    If key<>""
      sub= GetJSONMember(object,key)
    Else
      sub=object
    EndIf
    If sub And JSONType(sub)=#PB_JSON_Number
      ProcedureReturn GetJSONDouble(sub)
    EndIf
    ProcedureReturn def
  EndProcedure
  Procedure GetStringList(key.s,List string.s())
    If OpenArray(key)
      While NextArrayEntry()
        AddElement(string())
        string()=GetString()
      Wend      
      CloseArray()
      ProcedureReturn #True
    EndIf
    ProcedureReturn #False    
  EndProcedure  
  
  Procedure SetString(key.s,str.s)
    If key="" 
      SetJSONString(object,str)
    Else
      SetJSONString( AddJSONMember(object,key), str) 
    EndIf
  EndProcedure
  Procedure SetInteger(key.s,int)
    If key="" 
      SetJSONInteger(object,int)
    Else
      SetJSONInteger( AddJSONMember(object,key), int) 
    EndIf
  EndProcedure
  Procedure SetDouble(key.s,int.d)
    If key="" 
      SetJSONDouble(object,int)
    Else
      SetJSONDouble( AddJSONMember(object,key), int) 
    EndIf
  EndProcedure
  Procedure SetBool(key.s,int)
    If key="" 
      SetJSONBoolean(object,int)
    Else
      SetJSONBoolean( AddJSONMember(object,key), int) 
    EndIf
  EndProcedure
EndModule
; IDE Options = PureBasic 5.62 (Windows - x86)
; CursorPosition = 78
; FirstLine = 64
; Folding = ------
; EnableXP